@@ -3,10 +3,17 @@
 /*  Copyright (c) 1996 Brian J. Fox
     Author: Brian J. Fox (bfox@ai.mit.edu) Thu Aug 22 18:09:27 1996.  */
 
+/* Defined every time by configure. */
+#undef MHTML_SYSTEM_TYPE
+
+/* Defined on those systems for which ODBC compilation with the UDBC client
+   is desired. */
+#undef OPENLINK_UDBC_CLIENT
+
 /* Defined only when compiing on some brand of Solaris system. */
 #undef Solaris
 
-@BOT@
+@BOTTOM@
 #if defined (Solaris)
 #  include <arch/solaris/prototypes.h>
 #endif
