@@ -1,16 +1,30 @@
-;;; mhtml.el: -*- Emacs-Lisp -*-  DESCRIPTIVE TEXT.
-
+;;; mhtml.el: -*- Emacs-Lisp -*-  mode for the Meta-HTML language.
+;;;
 ;;; Copyright (c) 1995 Brian J. Fox
 ;;; Author: Brian J. Fox (bfox@ai.mit.edu) Sat Aug 26 18:18:13 1995.
 ;;;
 ;;; A first crack at mhtml mode.
 ;;;
 ;;; The base major mode for editing Meta-HTML code.
+;;;
+;;; To use this mode, put the following in your ~/.emacs file
+;;;
+;;; (autoload 'meta-html-mode "mhtml" "Edit files of Meta-HTML code." t)
+;;;
+;;; (if (assoc "\\.mhtml$" auto-mode-alist)
+;;;     (rplacd (assoc "\\.mhtml$" auto-mode-alist) 'meta-html-mode)
+;;;   (setq auto-mode-alist (cons '("\\.mhtml$" . meta-html-mode)
+;;;				  auto-mode-alist)))
+;;;
+;;; Then, place this file where Emacs can find it, like
+;;; /usr/local/share/emacs/site-lisp, or whatever.
+;;;
 ;;; Code:
 
 (if (assoc "\\.mhtml$" auto-mode-alist)
     (rplacd (assoc "\\.mhtml$" auto-mode-alist) 'meta-html-mode)
-  (setq auto-mode-alist (cons '("\\.mhtml$" . meta-html-mode) auto-mode-alist)))
+  (setq auto-mode-alist (cons '("\\.mhtml$" . meta-html-mode)
+			      auto-mode-alist)))
 
 (defvar mhtml-mode-syntax-table nil "")
 
@@ -22,7 +36,8 @@
 	   (modify-syntax-entry ?\( "-" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\) "-" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\; "w" mhtml-mode-syntax-table)
-	   (modify-syntax-entry ?\& "w" mhtml-mode-syntax-table)))
+	   (modify-syntax-entry ?\& "w" mhtml-mode-syntax-table)
+	   (modify-syntax-entry ?\| "w" mhtml-mode-syntax-table)))
 
 (defun mhtml-mode-variables (mhtml-syntax)
   (set-syntax-table mhtml-mode-syntax-table)
@@ -109,7 +124,7 @@
   "Major mode for editing Meta-HTML code.
 Commands:
 Delete converts tabs to spaces as it moves back.
-Blank lines separate paragraphs.  Semicolons start comments.
+Blank lines separate paragraphs.  Three semicolons start comments.
 \\{mhtml-mode-map}
 Entry to this mode calls the value of `mhtml-mode-hook'
 if that value is non-nil."
@@ -186,13 +201,15 @@ rigidly along with this one."
 
 (defconst mhtml-complex-tags
   '( "when" "while" "defsubst" "defun" "defmacro" "defstruct"
+     "define-tag" "define-function"
      "with-open-database" "with-open-stream" "server-push"
      "comment" "verbatim" "small-caps" "plain-text" "ptext" "section"
-     "ul" "ol" "table" "tr" "td" "center" "a" "form" "select" "caption"
+     "ul" "ol" "dl" "table" "tr" "td" "center" "a" "form" "select" "caption"
      "applet" "subtable" "in-package" "with-local-package"
      "example" "question" "nested-items-list" "nested-item" "slide" "bullet"
      "menu" "layout::page" "layout::left-border" "layout::right-border"
-     "frameset")
+     "frameset" "msql::with-open-database" "odbc::with-open-database"
+     "dbm::with-open-database" "def-pagelist")
   "List of tag starts which are complex.")
 
 (defconst mhtml-complex-tag-start-regex
@@ -439,6 +456,12 @@ of the start of the containing expression."
 (put 'database-query 'mhtml-indent-function 'defun)
 (put 'database-load-record 'mhtml-indent-function 'defun)
 (put 'database-save-record 'mhtml-indent-function 'defun)
+(put 'msql::database-query 'mhtml-indent-function 'defun)
+(put 'msql::database-load-record 'mhtml-indent-function 'defun)
+(put 'msql::database-save-record 'mhtml-indent-function 'defun)
+(put 'odbc::database-query 'mhtml-indent-function 'defun)
+(put 'odbc::database-load-record 'mhtml-indent-function 'defun)
+(put 'odbc::database-save-record 'mhtml-indent-function 'defun)
 (put 'if 'mhtml-indent-function 2)
 (put 'ifeq 'mhtml-indent-function 3)
 (put 'var-case 'mhtml-indent-function 'defun)
@@ -446,6 +469,23 @@ of the start of the containing expression."
 (put 'get-var 'mhtml-indent-function 'defun)
 (put 'set-session-var 'mhtml-indent-function 'defun)
 (put 'get-session-var 'mhtml-indent-function 'defun)
+(put 'PROG 'mhtml-indent-function 'defun)
+(put 'DATABASE-QUERY 'mhtml-indent-function 'defun)
+(put 'DATABASE-LOAD-RECORD 'mhtml-indent-function 'defun)
+(put 'DATABASE-SAVE-RECORD 'mhtml-indent-function 'defun)
+(put 'MSQL::DATABASE-QUERY 'mhtml-indent-function 'defun)
+(put 'MSQL::DATABASE-LOAD-RECORD 'mhtml-indent-function 'defun)
+(put 'MSQL::DATABASE-SAVE-RECORD 'mhtml-indent-function 'defun)
+(put 'ODBC::DATABASE-QUERY 'mhtml-indent-function 'defun)
+(put 'ODBC::DATABASE-LOAD-RECORD 'mhtml-indent-function 'defun)
+(put 'ODBC::DATABASE-SAVE-RECORD 'mhtml-indent-function 'defun)
+(put 'IF 'mhtml-indent-function 2)
+(put 'IFEQ 'mhtml-indent-function 3)
+(put 'VAR-CASE 'mhtml-indent-function 'defun)
+(put 'SET-VAR 'mhtml-indent-function 'defun)
+(put 'GET-VAR 'mhtml-indent-function 'defun)
+(put 'SET-SESSION-VAR 'mhtml-indent-function 'defun)
+(put 'GET-SESSION-VAR 'mhtml-indent-function 'defun)
 
 (defun mhtml-indent-sexp (&optional endpos)
   "Indent each line of the list starting just after point.
@@ -546,11 +586,32 @@ ENDPOS is encountered."
 (defun mhtml-indent-region (start end)
   (interactive "r")
   (save-excursion
-    (let ((endmark (copy-marker end)))
+    (let ((total-lines (count-lines start end))
+	  (which 0))
       (goto-char start)
-      (and (bolp) (not (eolp))
-	   (mhtml-indent-line))
-      (mhtml-indent-sexp endmark)
-      (set-marker endmark nil))))
+      (while (< which total-lines)
+	(beginning-of-line)
+	(delete-horizontal-space)
+	(if (not (eolp))
+	    (indent-for-tab-command 1))
+	(forward-line 1)
+	(setq which (+ which 1))))))
 
 (provide 'mhtml-mode)
+
+(autoload 'compile-internal "compile")
+
+(defun mhtml-compile-file (filename)
+  "Run mklib on a file (default: current buffer), and append .lib to
+the output filename."
+  (interactive
+   (list (read-file-name "Make Meta-HTML lib from source file: "
+                         (buffer-file-name)
+                         (buffer-file-name)
+                         t)))
+  (setq filename (expand-file-name filename))
+  (cd (file-name-directory filename))
+  (setq file (file-name-nondirectory filename))
+  (compile-internal
+   (concat "mklib -o " (file-name-sans-extension file) ".lib " file)
+   "No more errors."))
