@@ -43,6 +43,7 @@
 #include <regex.h>
 #include <setjmp.h>
 #include <sys/types.h>
+#include <errno.h>
 #include <time.h>
 #if defined (Solaris)
 #  include <ucbinclude/sys/fcntl.h>
@@ -76,6 +77,10 @@ extern char *strdup (const char *string);
 #  define os_open(name, flags, mode) open (name, flags, mode)
 #endif
 
+#if !defined (errno)
+extern int errno;
+#endif
+
 static void pf_with_open_database (PFunArgs);
 static void pf_database_load_record (PFunArgs);
 static void pf_database_delete_record (PFunArgs);
@@ -123,29 +128,6 @@ initialize_db_functions (Package *package)
     }
 }
 
-static int
-all_digits (char *string)
-{
-  register int i;
-  int result = 0;
-
-  /* Skip leading whitespace. */
-  for (i = 0; whitespace (string[i]); i++);
-
-  if (string[i])
-    {
-      result = 1;
-
-      for (; string[i]; i++)
-	if (!isdigit (string[i]))
-	  {
-	    result = 0;
-	    break;
-	  }
-    }
-  return (result);
-}
-
 static long
 random_number (void)
 {
@@ -178,6 +160,16 @@ random_number (void)
   return (value);
 }
 
+static void
+dbobj_free (DBOBJ *obj)
+{
+  if (obj != (DBOBJ *)NULL)
+    {
+      if (obj->data) free (obj->data);
+      free (obj);
+    }
+}
+
 /************************************************************/
 /*							    */
 /*		Database Manipulation Functions		    */
@@ -229,6 +221,8 @@ lock_database (char *dbname, int open_mode, int *lock, DBFILE *db)
 
   if ((fd < 0) || (LOCKFILE (fd) == -1))
     {
+      page_syserr ("%s: %s", dbname, (char *)strerror (errno));
+
       if (fd >= 0)
 	{
 	  char pid_name[100];
@@ -242,7 +236,8 @@ lock_database (char *dbname, int open_mode, int *lock, DBFILE *db)
     {
       if ((*db = database_open (dbname, open_mode)) == (DBFILE)0)
 	{
-	  unlink (lockname);
+	  page_syserr ("%s: %s", dbname, database_strerror ());
+	  /* unlink (lockname); */
 	  UNLOCKFILE (fd);
 	  close (fd);
 	}
@@ -281,12 +276,12 @@ pf_with_open_database (PFunArgs)
   int mode;
   int jump_again = 0;
 
-  varname = page_evaluate_string (get_positional_arg (vars, 0));
-  dbname = page_evaluate_string (get_positional_arg (vars, 1));
+  varname = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  dbname = mhtml_evaluate_string (get_positional_arg (vars, 1));
 
   if ((!empty_string_p (varname)) && (!empty_string_p (dbname)))
     {
-      char *modename = page_evaluate_string (get_value (vars, "MODE"));
+      char *modename = mhtml_evaluate_string (get_value (vars, "MODE"));
       DBFILE db;
       int lock;
 
@@ -350,8 +345,8 @@ dbkey_function_args (Package *vars, DBFILE *db, DBOBJ **key, int key_required)
   if (database_environment_level == 0)
     return;
 
-  dbref  = page_evaluate_string (get_positional_arg (vars, 0));
-  dbkey  = page_evaluate_string (get_positional_arg (vars, 1));
+  dbref  = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  dbkey  = mhtml_evaluate_string (get_positional_arg (vars, 1));
 
   if ((dbref != (char *)NULL) &&
       (!key_required || (dbkey != (char *)NULL)))
@@ -410,7 +405,7 @@ pf_database_load_record (PFunArgs)
   if ((db == (DBFILE)0) || (key == (DBOBJ *)NULL))
     return;
 
-  prefix = page_evaluate_string
+  prefix = mhtml_evaluate_string
     (get_one_of (vars, "PACKAGE", "PREFIX", (char *)NULL));
   prefix_len = prefix ? strlen (prefix) : 0;
 
@@ -486,18 +481,10 @@ pf_database_load_record (PFunArgs)
       result = "true";
     }
 
-  if (content)
-    {
-      if (content->data) free (content->data);
-      free (content);
-    }
+  dbobj_free (content);
+  dbobj_free (key);
 
   if (prefix) free (prefix);
-  if (key)
-    {
-      if (key->data) free (key->data);
-      free (key);
-    }
 
   if (result != (char *)NULL)
     bprintf_insert (page, start, "%s", result);
@@ -509,17 +496,25 @@ pf_database_delete_record (PFunArgs)
 {
   DBFILE db;
   DBOBJ *key;
+  char *result = (char *)NULL;
 
   dbkey_function_args (vars, &db, &key, 1);
 
   if (db && key)
-    database_delete (db, key);
-
-  if (key)
     {
-      if (key->data) free (key->data);
-      free (key);
+      database_delete (db, key);
+
+      if (database_had_error_p ())
+	page_syserr ("<database-delete-record %s>: %s",
+		     mhtml_funargs (vars), database_strerror ());
+      else
+	result = "true";
     }
+
+  dbobj_free (key);
+
+  if (result != (char *)NULL)
+    bprintf_insert (page, start, "%s", result);
 }
 
 /* <database-save-record db key var1 var2 ... varN [prefix=string]> */
@@ -535,7 +530,7 @@ pf_database_save_record (PFunArgs)
   if (db && key)
     {
       int position = 2;
-      char *prefix = page_evaluate_string
+      char *prefix = mhtml_evaluate_string
 	(get_one_of (vars, "PACKAGE", "PREFIX", (char *)NULL));
       int prefix_len = prefix ? strlen (prefix) : 0;
       char *name;
@@ -573,17 +568,15 @@ pf_database_save_record (PFunArgs)
 	content.length = (size_t)data->bindex;
 
 	database_store (db, key, &content);
+	if (database_had_error_p ())
+	  page_syserr ("%s", database_strerror ());
       }
       bprintf_free_buffer (data);
 
       result = "true";
     }
 
-  if (key)
-    {
-      if (key->data) free (key->data);
-      free (key);
-    }
+  dbobj_free (key);
 
   if (result)
     bprintf_insert (page, start, "%s", result);
@@ -602,7 +595,7 @@ pf_database_save_package (PFunArgs)
   if (db && key)
     {
       int strip = !empty_string_p (get_value (vars, "strip"));
-      char *package_name = page_evaluate_string (get_positional_arg (vars, 2));
+      char *package_name = mhtml_evaluate_string (get_positional_arg (vars, 2));
       Symbol **symbols = (Symbol **)NULL;
 
       if (!empty_string_p (package_name))
@@ -666,6 +659,9 @@ pf_database_save_package (PFunArgs)
 	    content.length = (size_t)data->bindex;
 
 	    database_store (db, key, &content);
+
+	    if (database_had_error_p ())
+	      page_syserr ("%s", database_strerror ());
 	  }
 
 	  bprintf_free_buffer (data);
@@ -673,11 +669,7 @@ pf_database_save_package (PFunArgs)
 	  result = "true";
 	}
 
-      if (key)
-	{
-	  if (key->data) free (key->data);
-	  free (key);
-	}
+      dbobj_free (key);
 
       if (result)
 	bprintf_insert (page, start, "%s", result);
@@ -702,8 +694,7 @@ pf_database_first_key (PFunArgs)
 	  bprintf_insert (page, start, "%s", key->data);
 	  *newstart = start + (key->length - 1);
 
-	  free (key->data);
-	  free (key);
+	  dbobj_free (key);
 	}
     }
 }
@@ -713,7 +704,8 @@ static void
 pf_database_next_key (PFunArgs)
 {
   DBFILE db;
-  DBOBJ *inkey, *outkey = (DBOBJ *)NULL;
+  DBOBJ *inkey = (DBOBJ *)NULL;
+  DBOBJ *outkey = (DBOBJ *)NULL;
 
   dbkey_function_args (vars, &db, &inkey, 1);
 
@@ -724,15 +716,10 @@ pf_database_next_key (PFunArgs)
     {
       bprintf_insert (page, start, "%s", outkey->data);
       *newstart = start + (outkey->length - 1);
-      free (outkey->data);
-      free (outkey);
+      dbobj_free (outkey);
     }
 
-  if (inkey)
-    {
-      free (inkey->data);
-      free (inkey);
-    }
+  dbobj_free (inkey);
 }
 
 /* <database-unique-key db [suggestion]> --> a unique key. */
@@ -782,10 +769,7 @@ pf_database_unique_key (PFunArgs)
 	      done = 1;
 	    }
 	  else
-	    {
-	      free (content->data);
-	      free (content);
-	    }
+	    dbobj_free (content);
 	}
       free (unique_id);
       free (suggestion);
@@ -890,7 +874,7 @@ static void
 pf_database_first_match (PFunArgs)
 {
   DBFILE db = (DBFILE)0;
-  char *dbref = page_evaluate_string (get_positional_arg (vars, 0));
+  char *dbref = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *field_expr;
   char *search_expr;
   int fields_only_p = var_present_p (vars, "fields-only");
@@ -898,8 +882,8 @@ pf_database_first_match (PFunArgs)
   if (database_environment_level == 0)
     return;
 
-  field_expr = page_evaluate_string (get_positional_arg (vars, 1));
-  search_expr= page_evaluate_string (get_positional_arg (vars, 2));
+  field_expr = mhtml_evaluate_string (get_positional_arg (vars, 1));
+  search_expr= mhtml_evaluate_string (get_positional_arg (vars, 2));
 
   if (search_expr == (char *)NULL)
     return;
@@ -930,10 +914,10 @@ static void
 pf_database_next_match (PFunArgs)
 {
   DBFILE db = (DBFILE)0;
-  char *dbref = page_evaluate_string (get_positional_arg (vars, 0));
-  char *dbkey = page_evaluate_string (get_positional_arg (vars, 1));
-  char *field_expr = page_evaluate_string (get_positional_arg (vars, 2));
-  char *search_expr = page_evaluate_string (get_positional_arg (vars, 3));
+  char *dbref = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  char *dbkey = mhtml_evaluate_string (get_positional_arg (vars, 1));
+  char *field_expr = mhtml_evaluate_string (get_positional_arg (vars, 2));
+  char *search_expr = mhtml_evaluate_string (get_positional_arg (vars, 3));
   int fields_only_p = var_present_p (vars, "fields-only");
 
   if (database_environment_level == 0)
@@ -1062,7 +1046,7 @@ dbrec_comp (const void *arg1, const void *arg2)
   else if (string1 && string2)
     {
       /* Check for both strings all digits.  If so, sort numerically. */
-      if (all_digits (string1) && all_digits (string2))
+      if (mhtml_all_digits (string1) && mhtml_all_digits (string2))
 	{
 	  long val1 = atol (string1);
 	  long val2 = atol (string2);
@@ -1120,7 +1104,7 @@ dbrec_callout (const void *arg1, const void *arg2)
 	}
 
       bprintf (buff, ">");
-      compare_result = page_evaluate_string (buff->buffer);
+      compare_result = mhtml_evaluate_string (buff->buffer);
       bprintf_free_buffer (buff);
 
       if (empty_string_p (compare_result))
@@ -1164,11 +1148,11 @@ static void
 pf_database_query (PFunArgs)
 {
   DBFILE db = (DBFILE)0;
-  char *dbref = page_evaluate_string (get_positional_arg (vars, 0));
+  char *dbref = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *expr = get_positional_arg (vars, 1);
   char *format_expr = get_value (vars, "format");
-  char *sort_args = page_evaluate_string (get_value (vars, "sort"));
-  char *sortfun = page_evaluate_string (get_value (vars, "predicate"));
+  char *sort_args = mhtml_evaluate_string (get_value (vars, "sort"));
+  char *sortfun = mhtml_evaluate_string (get_value (vars, "predicate"));
   char *keys_var = get_value (vars, "keys");
 
   if (database_environment_level == 0 || empty_string_p (dbref))
@@ -1199,7 +1183,7 @@ pf_database_query (PFunArgs)
       int search_limit = -1;
 
       search_limit_string =
-	page_evaluate_string (get_value (vars, "search-limit"));
+	mhtml_evaluate_string (get_value (vars, "search-limit"));
 
       if (!empty_string_p (search_limit_string))
 	{
@@ -1220,7 +1204,7 @@ pf_database_query (PFunArgs)
 	      char *expr_result;
 
 	      symbol_set_default_package (db_fields);
-	      expr_result = page_evaluate_string (expr);
+	      expr_result = mhtml_evaluate_string (expr);
 	      symbol_set_default_package (save);
 
 	      /* If satisfied, save this key. */
@@ -1248,11 +1232,7 @@ pf_database_query (PFunArgs)
 	  key = nextkey;
 	}
 
-      if (key != (DBOBJ *)NULL)
-	{
-	  free (key->data);
-	  free (key);
-	}
+      dbobj_free (key);
 
       /* If there are any matched keys, then sort, format, and/or return
 	 the keys. */
@@ -1318,7 +1298,7 @@ pf_database_query (PFunArgs)
 	      char *temp;
 	      char *fl;
 
-	      fl = page_evaluate_string (get_value (vars, "format-limit"));
+	      fl = mhtml_evaluate_string (get_value (vars, "format-limit"));
 	      if (!empty_string_p (fl))
 		{
 		  format_limit = atoi (fl);
@@ -1331,7 +1311,7 @@ pf_database_query (PFunArgs)
 			   ((format_limit < 0) || (i < format_limit))); i++)
 		{
 		  symbol_set_default_package (records[i]->contents);
-		  temp = page_evaluate_string (format_expr);
+		  temp = mhtml_evaluate_string (format_expr);
 		  symbol_set_default_package (save);
 
 		  if (temp)
@@ -1347,7 +1327,7 @@ pf_database_query (PFunArgs)
 	     place to put the keys for this record, then do so now. */
 	  if (keys_var != (char *)NULL)
 	    {
-	      char *sym_name = page_evaluate_string (keys_var);
+	      char *sym_name = mhtml_evaluate_string (keys_var);
 
 	      if (!empty_string_p (sym_name))
 		{
@@ -1408,7 +1388,7 @@ pf_database_query (PFunArgs)
 	     to no keys. */
 	  if (keys_var != (char *)NULL)
 	    {
-	      char *sym_name = page_evaluate_string (keys_var);
+	      char *sym_name = mhtml_evaluate_string (keys_var);
 
 	      if (!empty_string_p (sym_name))
 		{
