@@ -55,14 +55,14 @@
 #include "mac_port.h"
 #endif
 
-#define form_tag_ender(c) \
-	(((c) == '"') || ((c) == ' ') || \
-	 ((c) == '\t') || ((c) == '\n') || ((c) == '>'))
-
 #undef whitespace
 #define whitespace(c) \
      (((c) == ' ') || ((c) == '\t') || ((c) == '\r') || ((c) == '\n'))
 
+#define form_tag_ender(c) (whitespace (c) || (c == '>') || (c == '"'))
+#define tag_ender(c)	  (whitespace (c) || (c == '>'))
+
+
 #if !defined (os_open)
 #  define os_open(file, mode, flags) open (file, mode, flags)
 #if defined (Solaris)
@@ -530,6 +530,9 @@ page_set_form_input_value (PAGE *page, char *tag, char *value)
 /* This variable is where the output of calling page_debug_xxx gets stored. */
 static BPRINTF_BUFFER *debugger_output = (BPRINTF_BUFFER *)NULL;
 
+/* This variable is where the output of calling page_syserr gets stored. */
+static BPRINTF_BUFFER *syserr_output = (BPRINTF_BUFFER *)NULL;
+
 /* Like printf, but stores the output in debugger_output. */
 void
 page_debug (char *format, ...)
@@ -546,6 +549,22 @@ page_debug (char *format, ...)
     bprintf (debugger_output, "\n");
 }
 
+/* Like printf, but stores the output in syserr_output. */
+void
+page_syserr (char *format, ...)
+{
+  va_list args;
+
+  if (!syserr_output)
+    syserr_output = bprintf_create_buffer ();
+
+  va_start (args, format);
+
+  vbprintf (syserr_output, format, args);
+  if (syserr_output->buffer[syserr_output->bindex - 1] != '\n')
+    bprintf (syserr_output, "\n");
+}
+
 char *
 page_debug_buffer (void)
 {
@@ -555,6 +574,15 @@ page_debug_buffer (void)
   return ((char *)debugger_output->buffer);
 }
 
+char *
+page_syserr_buffer (void)
+{
+  if (!syserr_output || !syserr_output->bindex)
+    return ((char *)NULL);
+
+  return ((char *)syserr_output->buffer);
+}
+
 void
 page_debug_clear (void)
 {
@@ -564,6 +592,15 @@ page_debug_clear (void)
   debugger_output = (BPRINTF_BUFFER *)NULL;
 }
 
+void
+page_syserr_clear (void)
+{
+  if (syserr_output)
+    bprintf_free_buffer (syserr_output);
+
+  syserr_output = (BPRINTF_BUFFER *)NULL;
+}
+
 /* Given TICKS (the number of seconds since the epoch), return a string
    representing that date in the format the HTTP header mandate.  The
    string returned comes from a static buffer, the caller must manually
@@ -592,6 +629,7 @@ http_date_format (long ticks)
   return (result);
 }
 
+#if defined (PAGE_INSERT_HTTP_HEADER)
 /* Insert a stnadard HTTP header at the start of page if it isn't already
    a redirection specification.  EXPIRATION is one of:
 
@@ -684,7 +722,9 @@ page_insert_http_header (PAGE *page, int expiration)
       bprintf_free_buffer (http_header);
     }
 }
+#endif /* PAGE_INSERT_HTTP_HEADER */
 
+#if defined (PAGE_SET_COOKIE)
 /* Set a mime header in PAGE giving it NAME and VALUE as a HTTP-COOKIE.
    PAGE should already have a full Mime header in it. */
 void
@@ -716,7 +756,9 @@ page_set_cookie (PAGE *page, char *name, char *value, char *path)
       bprintf_insert (page, i, "%s\n", cookie_buffer->buffer);
     }
 }
+#endif /* PAGE_SET_COOKIE */
 
+#if defined (HTTP_RETURN_IMAGE)
 /* Given the pathname of a image file, return a page containing the image
    prefixed with the correct HTTP magic. */
 PAGE *
@@ -740,6 +782,7 @@ http_return_image (char *name, char *type)
     }
   return (page);
 }
+#endif /* HTTP_RETURN_IMAGE */
 
 /* Create a generic error page with placeholders for <HEADER>, <ERROR-MESSAGE>,
    <DEBUGGING-OUTPUT>, <RETURN-TO-URL> and <FOOTER>. */
@@ -751,7 +794,7 @@ page_error_page (void)
   page = bprintf_create_buffer ();
 
   bprintf (page, "<HEADER>\n<ERROR-MESSAGE>\n<RETURN-TO-URL>\n");
-  bprintf (page, "<DEBUGGING-OUTPUT><FOOTER>\n");
+  bprintf (page, "<DEBUGGING-OUTPUT><SYSTEM-ERROR-OUTPUT><FOOTER>\n");
   return (page);
 }
 
@@ -786,6 +829,23 @@ page_clean_up (PAGE *page)
       page_subst_in_page (page, "<DEBUGGING-OUTPUT>", subber->buffer);
       subber->bindex = 0;
     }
+  else
+    page_subst_in_page (page, "<DEBUGGING-OUTPUT>", "");
+
+  if (syserr_output != (BPRINTF_BUFFER *)NULL)
+    {
+      bprintf (subber, "%s", syserr_output->buffer);
+      page_subst_in_page (subber, "<", "&lt;");
+      page_subst_in_page (subber, ">", "&gt;");
+      page_subst_in_page (subber, "\n", "\n<br>");
+      bprintf_insert (subber, 0,
+		      "<h2>System Error Messages</h2><p>");
+
+      page_subst_in_page (page, "<SYSTEM-ERROR-OUTPUT>", subber->buffer);
+      subber->bindex = 0;
+    }
+  else
+    page_subst_in_page (page, "<SYSTEM-ERROR-OUTPUT>", "");
 
   if (page_search (page, "<FOOTER>", 0) != -1)
     page_subst_in_page (page, "<FOOTER>", "</body></html>\n");
@@ -799,32 +859,24 @@ page_clean_up (PAGE *page)
 int
 page_find_tag_start (PAGE *page, char *tag, int start)
 {
-  static char searcher[200];
   int point = -1;
+  register int i, c;
+  int len = strlen (tag), limit = page->bindex - (len + 1);
 
-  searcher[0] = '\0';
-
-  if ((tag != (char *)NULL) && (*tag != '\0') && (strlen (tag) < 190))
+  for (i = start; i < limit; i++)
     {
-      searcher[0] = '<';
-      strcpy (searcher + 1, tag);
-      strcat (searcher, "[ \t\r\n>]");
+      if (page->buffer[i] == '<')
+	{
+	  c = page->buffer[i + 1 + len];
+
+	  if (tag_ender (c) &&
+	      (strncasecmp (page->buffer + i + 1, tag, len) == 0))
+	    {
+	      point = i;
+	      break;
+	    }
+	}
     }
-
-  if (searcher[0])
-    {
-      regex_t re;
-      regmatch_t offsets[2];
-      int matched;
-
-      regcomp (&re, searcher, REG_EXTENDED | REG_ICASE);
-      matched = regexec (&re, page->buffer + start, 1, offsets, 0) == 0;
-      regfree (&re);
-
-      if (matched)
-	point = offsets[0].rm_so + start;
-    }
-
   return (point);
 }
 
@@ -1008,6 +1060,7 @@ page_complex_tag_bounds (PAGE *page, char *tag, int *startp, int *endp)
   return (result);
 }
 
+#if defined (NEVER_USED)
 /* Extract the value side of a randomly named variable in PAGE.
    For example, if PAGE contains `<DOCTITLE="this">', then the
    call: page_assigned_label (page, "DOCTITLE") returns "this". */
@@ -1058,6 +1111,7 @@ page_assigned_label (PAGE *page, char *tag)
   bprintf_free_buffer (search);
   return (value);
 }
+#endif /* NEVER_USED */
 
 /* Find and extract the inclusive boundaries in PAGE of a complex tag
    named TAG.  Returns the boundaries in STARTP and ENDP, and a string
