@@ -1,4 +1,4 @@
-/* prims.c: -*- C -*-  DESCRIPTIVE TEXT. */
+/* prims.c: -*- C -*-  Primitive internal functions. */
 
 /* Author: Brian J. Fox (bfox@ai.mit.edu) Sat Jul 20 17:22:47 1996.
 
@@ -59,20 +59,25 @@ extern int open (char *, int, ...);
 #include "forms.h"
 #include "session.h"
 #include "pages.h"
-#include "pagefuncs.h"
 #include "parser.h"
 #include "symdump.h"
 
+#if defined (HAVE_GETPWNAM)
+#  include <pwd.h>
+#endif
+
 static void pf_eval (PFunArgs);
 static void pf_read (PFunArgs);
 static void pf_write_package_file (PFunArgs);
 static void pf_read_package_file (PFunArgs);
+static void pf_switch_user (PFunArgs);
 
 PFunDesc prims_table[] = {
   { "%%EVAL",			0, 0, pf_eval },
   { "%%READ",			0, 0, pf_read },
   { "%%WRITE-PACKAGE-FILE",	0, 0, pf_write_package_file },
   { "%%READ-PACKAGE-FILE",	0, 0, pf_read_package_file },
+  { "%%SWITCH-USER",		0, 0, pf_switch_user },
   { (char *)NULL,		0, 0, (PFunHandler *)NULL }
 };
 
@@ -98,8 +103,8 @@ pf_eval (PFunArgs)
 
   if (result != (char *)NULL)
     {
-      char *expr = page_evaluate_string (result);
-      result = page_evaluate_string (expr);
+      char *expr = mhtml_evaluate_string (result);
+      result = mhtml_evaluate_string (expr);
 
       if (!empty_string_p (result))
 	{
@@ -136,7 +141,7 @@ pf_read (PFunArgs)
 static void
 pf_write_package_file (PFunArgs)
 {
-  char *filename = page_evaluate_string (get_positional_arg (vars, 0));
+  char *filename = mhtml_evaluate_string (get_positional_arg (vars, 0));
   int packages_written = 0;
 
   if (!empty_string_p (filename))
@@ -150,7 +155,7 @@ pf_write_package_file (PFunArgs)
 
 	  while ((arg = get_positional_arg (vars, i)) != (char *)NULL)
 	    {
-	      char *packname = page_evaluate_string (arg);
+	      char *packname = mhtml_evaluate_string (arg);
 
 	      if (!empty_string_p (packname))
 		{
@@ -179,7 +184,7 @@ pf_write_package_file (PFunArgs)
 static void
 pf_read_package_file (PFunArgs)
 {
-  char *filename = page_evaluate_string (get_positional_arg (vars, 0));
+  char *filename = mhtml_evaluate_string (get_positional_arg (vars, 0));
   int packages_read = 0;
 
   if (!empty_string_p (filename))
@@ -228,7 +233,32 @@ pf_read_package_file (PFunArgs)
     }
 }
 
+static void
+pf_switch_user (PFunArgs)
+{
+  char *username = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  char *result = (char *)NULL;
 
+#if defined (HAVE_GETPWNAM)
+  if (!empty_string_p (username))
+    {
+      struct passwd *entry = (struct passwd *)getpwnam (username);
 
+      if (entry != (struct passwd *)NULL)
+	{
+	  uid_t uid = (uid_t)entry->pw_uid;
 
-	  
+	  if (setuid (uid) != -1)
+	    result = "true";
+	}
+    }
+#endif /* HAVE_GETPWNAM */
+
+  if (username != (char *)NULL) free (username);
+
+  if (result != (char *)NULL)
+    {
+      bprintf_insert (page, start, "%s", result);
+      *newstart += strlen (result);
+    }
+}
