@@ -61,7 +61,6 @@ extern int open (char *, int, ...);
 #include "forms.h"
 #include "session.h"
 #include "pages.h"
-#include "pagefuncs.h"
 #include "parser.h"
 #include "symdump.h"
 
@@ -129,6 +128,8 @@ require_search (char *root, char *leaf, int *filetype)
   int nsize = strlen (root) + strlen (leaf) + 1024;
   struct stat finfo;
   char *result = (char *)NULL;
+  char *incpref = pagefunc_get_variable ("mhtml::include-prefix");
+  char *relpref = pagefunc_get_variable ("mhtml::relative-prefix");
 
   if (nsize >= psize)
     pathname = (char *)xrealloc (pathname, psize = nsize);
@@ -144,46 +145,10 @@ require_search (char *root, char *leaf, int *filetype)
       strcat (pathname, leaf);
     }
 
-  temp = pagefunc_canonicalize_file_name (pathname);
+  temp = mhtml_canonicalize_file_name (pathname, incpref, relpref);
 
-  /* Clean up the result so that "." and ".." disappear. */
-  if (temp)
-    {
-      register int last_slash = 0;
-
-      for (i = 0; temp[i] != '\0'; i++)
-	{
-	  /* If in eligible spot for "./" or "../" removal, do it now. */
-	  if ((i == 0) || (temp[i] == '/'))
-	    {
-	      if ((temp[i + 1] == '.') && (temp[i + 2] == '/'))
-		{
-		  /* Remove "./". */
-		  memmove (temp + i, temp + i + 2, 1 + strlen (temp + i + 2));
-		  i--;
-		}
-	      else if ((temp[i + 1] == '.') &&
-		       (temp[i + 2] == '.') &&
-	               (temp[i + 3] == '/'))
-		{
-		  /* Remove "../" back to previous slash location. */
-		  memmove (temp + last_slash, temp + i + 3,
-			   1 + strlen (temp + i + 3));
-		  i = last_slash - 1;
-
-		  /* Move the last slash back. */
-		  for (last_slash = i; last_slash > 0; last_slash--)
-		    if (temp[last_slash] == '/')
-		      break;
-		}
-	      else
-		last_slash = i;
-	    }
-	}
-    }
-
-  /* If found, return it immediately. */
-  if (stat (temp, &finfo) != -1)
+  /* If found and not a directory, return it immediately. */
+  if ((stat (temp, &finfo) != -1) && (!S_ISDIR (finfo.st_mode)))
     {
       result = temp;
     }
@@ -260,7 +225,7 @@ static void
 pf_require (PFunArgs)
 {
   register int i;
-  char *arg = page_evaluate_string (get_positional_arg (vars, 0));
+  char *arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char **directories = symbol_get_values (REQUIRE_DIRECTORIES);
   char *default_dirs[2] = { ".", (char *)NULL };
   char *pathname = (char *)NULL;
@@ -338,6 +303,9 @@ pf_require (PFunArgs)
 	      {
 		require_remember_pathname (pathname);
 		while ((pack = symbol_load_package (fd)) != (Package *)NULL);
+		if (mhtml_user_keywords == (Package *)NULL)
+		  mhtml_user_keywords = 
+		    symbol_lookup_package ("*user-functions*");
 		close (fd);
 	      }
 	    else
