@@ -347,7 +347,7 @@ static void
 pf_sessions_of_key (PFunArgs)
 {
   register int i;
-  char *arg = page_evaluate_string (get_positional_arg (vars, 0));
+  char *arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
   SESSION_INFO **info_list = session_all_sessions ();
   SESSION_INFO *info;
 
@@ -395,10 +395,10 @@ pf_set_session_var (PFunArgs)
 	    {
 	      char *name = names[i], *value = vals[i];
 
-	      name = page_evaluate_string (name);
+	      name = mhtml_evaluate_string (name);
 
 	      if (value != (char *)NULL)
-		value = page_evaluate_string (value);
+		value = mhtml_evaluate_string (value);
 
 	      forms_set_tag_value_in_package (session_data, name, value);
 	      if (value) free (value);
@@ -436,7 +436,7 @@ pf_get_session_var (PFunArgs)
 	    {
 	      char *name = names[i], *value = (char *)NULL;
 
-	      name = page_evaluate_string (name);
+	      name = mhtml_evaluate_string (name);
 
 	      value = forms_get_tag_value_in_package (session_data, name);
 
@@ -458,7 +458,7 @@ pf_get_session_var (PFunArgs)
 static void
 pf_get_session_data (PFunArgs)
 {
-  char *passed = page_evaluate_string (get_positional_arg (vars, 0));
+  char *passed = mhtml_evaluate_string (get_positional_arg (vars, 0));
   session_id_t sid;
 
   if (!empty_string_p (passed))
@@ -491,7 +491,7 @@ valid_alist_p (char *alist)
 static void
 pf_set_session_data (PFunArgs)
 {
-  char *alist = page_evaluate_string (get_positional_arg (vars, 0));
+  char *alist = mhtml_evaluate_string (get_positional_arg (vars, 0));
   session_id_t sid;
 
   sid = (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
@@ -546,7 +546,7 @@ pf_unset_session_var (PFunArgs)
 	    {
 	      char *name = names[i];
 
-	      name = page_evaluate_string (name);
+	      name = mhtml_evaluate_string (name);
 
 	      if (name != (char *)NULL)
 		symbol_remove_in_package (session_data, name);
@@ -578,8 +578,8 @@ pf_session_export (PFunArgs)
 	  Package *package, *session_data;
 	  Symbol *sym, **symbols = (Symbol **)NULL;
 
-	  internal_name = page_evaluate_string (get_positional_arg (vars, 0));
-	  external_name = page_evaluate_string (get_positional_arg (vars, 1));
+	  internal_name = mhtml_evaluate_string (get_positional_arg (vars, 0));
+	  external_name = mhtml_evaluate_string (get_positional_arg (vars, 1));
 	  session_data = symbol_get_package ((char *)NULL);
 	  sd_info_to_package (info, session_data);
 
@@ -658,8 +658,8 @@ pf_session_import (PFunArgs)
       if (symbols != (Symbol **)NULL)
 	{
 	  register int i;
-	  char *external = page_evaluate_string (get_positional_arg (vars, 0));
-	  char *internal = page_evaluate_string (get_positional_arg (vars, 1));
+	  char *external = mhtml_evaluate_string (get_positional_arg (vars, 0));
+	  char *internal = mhtml_evaluate_string (get_positional_arg (vars, 1));
 	  int external_len = (external ? strlen (external) : 0);
 	  int internal_len = (internal ? strlen (internal) : 0);
 	  Symbol *sym;
@@ -750,7 +750,7 @@ pf_session_import (PFunArgs)
 static void
 pf_set_session_timeout (PFunArgs)
 {
-  char *timeout_ascii = page_evaluate_string (get_positional_arg (vars, 0));
+  char *timeout_ascii = mhtml_evaluate_string (get_positional_arg (vars, 0));
   session_id_t sid =
     (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
 
