@@ -57,12 +57,17 @@
 #include <setjmp.h>
 #include <bprintf/bprintf.h>
 #include <xmalloc/xmalloc.h>
+#include <errno.h>
 #include "forms.h"
 #include "session.h"
 #include "pages.h"
 #include "parser.h"
 #include "streamfuncs.h"
 
+#if !defined (errno)
+extern int errno;
+#endif
+
 #if defined (Solaris)
 extern int open (char *, int, ...);
 #endif
@@ -227,16 +232,16 @@ int mhtml_stderr_fileno = 2;
 static void
 pf_with_open_stream (PFunArgs)
 {
-  char *varname = page_evaluate_string (get_positional_arg (vars, 0));
-  char *filename = page_evaluate_string (get_positional_arg (vars, 1));
+  char *varname = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  char *filename = mhtml_evaluate_string (get_positional_arg (vars, 1));
   int jump_again = 0;
 
   jump_again = 0;
 
   if ((!empty_string_p (varname)) && (!empty_string_p (filename)))
     {
-      char *modename = page_evaluate_string (get_value (vars, "MODE"));
-      char *typename = page_evaluate_string (get_value (vars, "TYPE"));
+      char *modename = mhtml_evaluate_string (get_value (vars, "MODE"));
+      char *typename = mhtml_evaluate_string (get_value (vars, "TYPE"));
       int mode = stream_mode (modename);
       int type = stream_type (typename);
       int flags = 0;
@@ -312,6 +317,15 @@ pf_with_open_stream (PFunArgs)
 
 	  discard_stream (page_value);
 	}
+      else
+	{
+	  char *temp = mhtml_funargs (vars);
+
+	  page_syserr ("<with-open-stream%s%s>: %s",
+		       temp ? " " : "", temp ? temp : "",
+		       (char *)strerror (errno));
+	  if (temp) free (temp);
+	}
     }
 
   if (filename) free (filename);
@@ -326,7 +340,7 @@ get_stream_reference (Package *vars)
 
   if (stream_environment_level != 0)
     {
-      char *stream_ref = page_evaluate_string (get_positional_arg (vars, 0));
+      char *stream_ref = mhtml_evaluate_string (get_positional_arg (vars, 0));
 
       if (stream_ref)
 	{
@@ -609,7 +623,7 @@ pf_stream_put (PFunArgs)
 
   if (stream && stream->type != stream_CLOSED)
     {
-      char *put_text = page_evaluate_string (get_positional_arg (vars, 1));
+      char *put_text = mhtml_evaluate_string (get_positional_arg (vars, 1));
 
       if (put_text)
 	{
@@ -627,7 +641,7 @@ pf_stream_put_contents (PFunArgs)
 
   if (stream && stream->type != stream_CLOSED)
     {
-      char *name = page_evaluate_string (get_positional_arg (vars, 1));
+      char *name = mhtml_evaluate_string (get_positional_arg (vars, 1));
       Symbol *sym = (name ? symbol_lookup (name) : (Symbol *)NULL);
 
       if (sym != (Symbol *)NULL)
@@ -670,7 +684,7 @@ static void
 pf_stream_get (PFunArgs)
 {
   Stream *stream = get_stream_reference (vars);
-  char *stop_at = page_evaluate_string (get_value (vars, "stop-at"));
+  char *stop_at = mhtml_evaluate_string (get_value (vars, "stop-at"));
   char *result = (char *)NULL;
 
   if (stream)
@@ -696,9 +710,9 @@ static void
 pf_stream_get_contents (PFunArgs)
 {
   Stream *stream = get_stream_reference (vars);
-  char *stop_at = page_evaluate_string (get_value (vars, "stop-at"));
-  char *chunk_size_arg = page_evaluate_string (get_value (vars, "chunk-size"));
-  char *name = page_evaluate_string (get_positional_arg (vars, 1));
+  char *stop_at = mhtml_evaluate_string (get_value (vars, "stop-at"));
+  char *chunk_size_arg = mhtml_evaluate_string (get_value (vars, "chunk-size"));
+  char *name = mhtml_evaluate_string (get_positional_arg (vars, 1));
   char *result = (char *)NULL;
   Symbol *sym;
 
