@@ -68,36 +68,36 @@ static char u_stb[128] =
 {
   0x00, /* nul */  0x01, /* soh */  0x02, /* stx */  0x03, /* etx */
   0x04, /* eot */  0x05, /* enq */  0x06, /* ack */  0x07, /* bel */
-  0x08, /* bs */  0x09, /* ht */  0x0a, /* nl */  0x0b, /* vt */
-  0x0c, /* np */  0x0d, /* cr */  0x0e, /* so */  0x0f, /* si */
+  0x08, /* bs */   0x09, /* ht */   0x0a, /* nl */   0x0b, /* vt */
+  0x0c, /* np */   0x0d, /* cr */   0x0e, /* so */   0x0f, /* si */
   0x10, /* dle */  0x11, /* dc1 */  0x12, /* dc2 */  0x13, /* dc3 */
   0x14, /* dc4 */  0x15, /* nak */  0x16, /* syn */  0x17, /* etb */
-  0x18, /* can */  0x19, /* em */  0x1a, /* sub */  0x1b, /* esc */
-  0x1c, /* fs */  0x1d, /* gs */  0x1e, /* rs */  0x1f, /* us */
-  0x20, /* sp */  0x21, /* ! */  0x22, /* " */  0x23, /* # */
-  0x24, /* $ */  0x25, /* % */  0x26, /* & */  0x27, /* ' */
-  0x28, /* ( */  0x29, /* ) */  0x2a, /* * */  0x2b, /* + */
-  0x2c, /* , */  0x2d, /* - */  0x2e, /* . */  0x2f, /* / */
-  0x30, /* 0 */  0x31, /* 1 */  0x32, /* 2 */  0x33, /* 3 */
-  0x34, /* 4 */  0x35, /* 5 */  0x36, /* 6 */  0x37, /* 7 */
-  0x38, /* 8 */  0x39, /* 9 */  0x3a, /* : */  0x3b, /* ; */
-  0x3c, /* < */  0x3d, /* = */  0x3e, /* > */  0x3f, /* ? */
-  0x40, /* @ */  0x41, /* A */  0x42, /* B */  0x43, /* C */
-  0x44, /* D */  0x45, /* E */  0x46, /* F */  0x47, /* G */
-  0x48, /* H */  0x49, /* I */  0x4a, /* J */  0x4b, /* K */
-  0x4c, /* L */  0x4d, /* M */  0x4e, /* N */  0x4f, /* O */
-  0x50, /* P */  0x51, /* Q */  0x52, /* R */  0x53, /* S */
-  0x54, /* T */  0x55, /* U */  0x56, /* V */  0x57, /* W */
-  0x58, /* X */  0x59, /* Y */  0x5a, /* Z */  0x5b, /* [ */
-  0x5c, /* \ */  0x5d, /* ] */  0x5e, /* ^ */  0x5f, /* _ */
-  0x60, /* ` */  0x41, /* A */  0x42, /* B */  0x43, /* C */
-  0x44, /* D */  0x45, /* E */  0x46, /* F */  0x47, /* G */
-  0x48, /* H */  0x49, /* I */  0x4a, /* J */  0x4b, /* K */
-  0x4c, /* L */  0x4d, /* M */  0x4e, /* N */  0x4f, /* O */
-  0x50, /* P */  0x51, /* Q */  0x52, /* R */  0x53, /* S */
-  0x54, /* T */  0x55, /* U */  0x56, /* V */  0x57, /* W */
-  0x58, /* X */  0x59, /* Y */  0x5a, /* Z */  0x7b, /* { */
-  0x7c, /* | */  0x7d, /* } */  0x7e, /* ~ */  0x7f /*del*/
+  0x18, /* can */  0x19, /* em */   0x1a, /* sub */  0x1b, /* esc */
+  0x1c, /* fs */   0x1d, /* gs */   0x1e, /* rs */   0x1f, /* us */
+  0x20, /* sp */   0x21, /* ! */    0x22, /* " */    0x23, /* # */
+  0x24, /* $ */    0x25, /* % */    0x26, /* & */    0x27, /* ' */
+  0x28, /* ( */    0x29, /* ) */    0x2a, /* * */    0x2b, /* + */
+  0x2c, /* , */    0x2d, /* - */    0x2e, /* . */    0x2f, /* / */
+  0x30, /* 0 */    0x31, /* 1 */    0x32, /* 2 */    0x33, /* 3 */
+  0x34, /* 4 */    0x35, /* 5 */    0x36, /* 6 */    0x37, /* 7 */
+  0x38, /* 8 */    0x39, /* 9 */    0x3a, /* : */    0x3b, /* ; */
+  0x3c, /* < */    0x3d, /* = */    0x3e, /* > */    0x3f, /* ? */
+  0x40, /* @ */    0x41, /* A */    0x42, /* B */    0x43, /* C */
+  0x44, /* D */    0x45, /* E */    0x46, /* F */    0x47, /* G */
+  0x48, /* H */    0x49, /* I */    0x4a, /* J */    0x4b, /* K */
+  0x4c, /* L */    0x4d, /* M */    0x4e, /* N */    0x4f, /* O */
+  0x50, /* P */    0x51, /* Q */    0x52, /* R */    0x53, /* S */
+  0x54, /* T */    0x55, /* U */    0x56, /* V */    0x57, /* W */
+  0x58, /* X */    0x59, /* Y */    0x5a, /* Z */    0x5b, /* [ */
+  0x5c, /* \ */    0x5d, /* ] */    0x5e, /* ^ */    0x5f, /* _ */
+  0x60, /* ` */    0x41, /* A */    0x42, /* B */    0x43, /* C */
+  0x44, /* D */    0x45, /* E */    0x46, /* F */    0x47, /* G */
+  0x48, /* H */    0x49, /* I */    0x4a, /* J */    0x4b, /* K */
+  0x4c, /* L */    0x4d, /* M */    0x4e, /* N */    0x4f, /* O */
+  0x50, /* P */    0x51, /* Q */    0x52, /* R */    0x53, /* S */
+  0x54, /* T */    0x55, /* U */    0x56, /* V */    0x57, /* W */
+  0x58, /* X */    0x59, /* Y */    0x5a, /* Z */    0x7b, /* { */
+  0x7c, /* | */    0x7d, /* } */    0x7e, /* ~ */    0x7f /*del*/
 };
 
 #if defined (LowerCaseTranslationTable)
@@ -107,36 +107,36 @@ static char l_stb[128] =
 {
   0x00, /* nul */  0x01, /* soh */  0x02, /* stx */  0x03, /* etx */
   0x04, /* eot */  0x05, /* enq */  0x06, /* ack */  0x07, /* bel */
-  0x08, /* bs */  0x09, /* ht */  0x0a, /* nl */  0x0b, /* vt */
-  0x0c, /* np */  0x0d, /* cr */  0x0e, /* so */  0x0f, /* si */
+  0x08, /* bs */   0x09, /* ht */   0x0a, /* nl */   0x0b, /* vt */
+  0x0c, /* np */   0x0d, /* cr */   0x0e, /* so */   0x0f, /* si */
   0x10, /* dle */  0x11, /* dc1 */  0x12, /* dc2 */  0x13, /* dc3 */
   0x14, /* dc4 */  0x15, /* nak */  0x16, /* syn */  0x17, /* etb */
-  0x18, /* can */  0x19, /* em */  0x1a, /* sub */  0x1b, /* esc */
-  0x1c, /* fs */  0x1d, /* gs */  0x1e, /* rs */  0x1f, /* us */
-  0x20, /* sp */  0x21, /* ! */  0x22, /* " */  0x23, /* # */
-  0x24, /* $ */  0x25, /* % */  0x26, /* & */  0x27, /* ' */
-  0x28, /* ( */  0x29, /* ) */  0x2a, /* * */  0x2b, /* + */
-  0x2c, /* , */  0x2d, /* - */  0x2e, /* . */  0x2f, /* / */
-  0x30, /* 0 */  0x31, /* 1 */  0x32, /* 2 */  0x33, /* 3 */
-  0x34, /* 4 */  0x35, /* 5 */  0x36, /* 6 */  0x37, /* 7 */
-  0x38, /* 8 */  0x39, /* 9 */  0x3a, /* : */  0x3b, /* ; */
-  0x3c, /* < */  0x3d, /* = */  0x3e, /* > */  0x3f, /* ? */
-  0x40, /* @ */  0x61, /* a */  0x62, /* b */  0x63, /* c */
-  0x64, /* d */  0x65, /* e */  0x66, /* f */  0x67, /* g */
-  0x68, /* h */  0x69, /* i */  0x6a, /* j */  0x6b, /* k */
-  0x6c, /* l */  0x6d, /* m */  0x6e, /* n */  0x6f, /* o */
-  0x70, /* p */  0x71, /* q */  0x72, /* r */  0x73, /* s */
-  0x74, /* t */  0x75, /* u */  0x76, /* v */  0x77, /* w */
-  0x78, /* x */  0x79, /* y */  0x7a, /* z */  0x5b, /* [ */
-  0x5c, /* \ */  0x5d, /* ] */  0x5e, /* ^ */  0x5f, /* _ */
-  0x60, /* ` */  0x61, /* a */  0x62, /* b */  0x63, /* c */
-  0x64, /* d */  0x65, /* e */  0x66, /* f */  0x67, /* g */
-  0x68, /* h */  0x69, /* i */  0x6a, /* j */  0x6b, /* k */
-  0x6c, /* l */  0x6d, /* m */  0x6e, /* n */  0x6f, /* o */
-  0x70, /* p */  0x71, /* q */  0x72, /* r */  0x73, /* s */
-  0x74, /* t */  0x75, /* u */  0x76, /* v */  0x77, /* w */
-  0x78, /* x */  0x79, /* y */  0x7a, /* z */  0x7b, /* { */
-  0x7c, /* | */  0x7d, /* } */  0x7e, /* ~ */  0x7f /*del*/
+  0x18, /* can */  0x19, /* em */   0x1a, /* sub */  0x1b, /* esc */
+  0x1c, /* fs */   0x1d, /* gs */   0x1e, /* rs */   0x1f, /* us */
+  0x20, /* sp */   0x21, /* ! */    0x22, /* " */    0x23, /* # */
+  0x24, /* $ */    0x25, /* % */    0x26, /* & */    0x27, /* ' */
+  0x28, /* ( */    0x29, /* ) */    0x2a, /* * */    0x2b, /* + */
+  0x2c, /* , */    0x2d, /* - */    0x2e, /* . */    0x2f, /* / */
+  0x30, /* 0 */    0x31, /* 1 */    0x32, /* 2 */    0x33, /* 3 */
+  0x34, /* 4 */    0x35, /* 5 */    0x36, /* 6 */    0x37, /* 7 */
+  0x38, /* 8 */    0x39, /* 9 */    0x3a, /* : */    0x3b, /* ; */
+  0x3c, /* < */    0x3d, /* = */    0x3e, /* > */    0x3f, /* ? */
+  0x40, /* @ */    0x61, /* a */    0x62, /* b */    0x63, /* c */
+  0x64, /* d */    0x65, /* e */    0x66, /* f */    0x67, /* g */
+  0x68, /* h */    0x69, /* i */    0x6a, /* j */    0x6b, /* k */
+  0x6c, /* l */    0x6d, /* m */    0x6e, /* n */    0x6f, /* o */
+  0x70, /* p */    0x71, /* q */    0x72, /* r */    0x73, /* s */
+  0x74, /* t */    0x75, /* u */    0x76, /* v */    0x77, /* w */
+  0x78, /* x */    0x79, /* y */    0x7a, /* z */    0x5b, /* [ */
+  0x5c, /* \ */    0x5d, /* ] */    0x5e, /* ^ */    0x5f, /* _ */
+  0x60, /* ` */    0x61, /* a */    0x62, /* b */    0x63, /* c */
+  0x64, /* d */    0x65, /* e */    0x66, /* f */    0x67, /* g */
+  0x68, /* h */    0x69, /* i */    0x6a, /* j */    0x6b, /* k */
+  0x6c, /* l */    0x6d, /* m */    0x6e, /* n */    0x6f, /* o */
+  0x70, /* p */    0x71, /* q */    0x72, /* r */    0x73, /* s */
+  0x74, /* t */    0x75, /* u */    0x76, /* v */    0x77, /* w */
+  0x78, /* x */    0x79, /* y */    0x7a, /* z */    0x7b, /* { */
+  0x7c, /* | */    0x7d, /* } */    0x7e, /* ~ */    0x7f /*  del*/
 };
 #endif /* LowerCaseTranslationTable */
 
@@ -754,24 +754,25 @@ symbol_free (Symbol *symbol)
       if (symbol->notifier)
 	*(symbol->notifier) = 0;
 
-      free (symbol->name);
 
       switch (symbol->type)
 	{
 	case symtype_STRING:
+	  free (symbol->name);
 	  free_array (symbol->values);
+	  free (symbol);
 	  break;
 
 	case symtype_BINARY:
+	  free (symbol->name);
 	  datablock_free ((Datablock *)symbol->values);
+	  free (symbol);
 	  break;
 
 	case symtype_FUNCTION:
 	case symtype_USERFUN:
 	  break;
 	}
-	  
-      free (symbol);
     }
 }
 
@@ -823,6 +824,7 @@ symbol_copy (Symbol *symbol, Package *package)
 
   copy = symbol_intern_in_package (package, symbol->name);
   copy->type = symbol->type;
+  copy->flags = symbol->flags;
   copy->values_index = symbol->values_index;
   copy->values_slots = symbol->values_slots;
 
@@ -957,6 +959,30 @@ symbol_copy_package (Package *from, Package *to)
     }
 }
 
+/* Store into the symbol referenced by NAME, the array of character strings
+   pointed to by ARRAY. */
+void
+symbol_store_array (char *name, char **array)
+{
+  Symbol *sym;
+
+  if ((sym = symbol_remove (name)) != (Symbol *)NULL)
+    symbol_free (sym);
+
+  /* Only assign when there really is anything to assign. */
+  if (array != (char **)NULL)
+    {
+      register int i;
+
+      for (i = 0; array[i] != (char *)NULL; i++);
+
+      sym = symbol_intern (name);
+      sym->values = array;
+      sym->values_index = i;
+      sym->values_slots = i;
+    }
+}
+
 #if defined (TEST)
 
 int
