@@ -57,6 +57,7 @@ typedef struct
 #define symtype_FUNCTION  0x01	/* Value points to a function definition. */
 #define symtype_BINARY    0x02	/* Value points to a datablock. */
 #define symtype_USERFUN	  0x03	/* Value is a user defined function. */
+#define symtype_ALIAS	  0x04	/* This symbol shares it's value. */
 
 /* Random data can be stored within. */
 typedef struct
@@ -203,6 +204,10 @@ extern Symbol **symbols_of_package (Package *package);
 /* Associate SYMBOL with NOTIFIER, the address of an integer. */
 extern void symbol_notify_value (Symbol *symbol, int *address);
 
+/* Store into the symbol referenced by NAME, the array of character strings
+   pointed to by ARRAY. */
+extern void symbol_store_array (char *name, char **array);
+
 extern int symbol_get_flags (Symbol *symbol);
 extern int symbol_get_flag (Symbol *symbol, int flag);
 extern void symbol_set_flag (Symbol *symbol, int flag);
