@@ -76,6 +76,8 @@
 extern int errno;
 #endif
 
+extern char **environ;
+
 static void mhttpd_fill_in_missing_headers (HTTP_RESULT *result);
 static PAGE *mhttpd_find_page_for_result (HTTP_RESULT *result);
 static void mhttpd_handle_empty_page (HTTP_RESULT *result);
@@ -140,11 +142,16 @@ mime_headers_from_string (char *string, int *last_char)
 	{
 	  if (string[i] == ':')
 	    colon_seen++;
+	  else if (!colon_seen && (!isalpha (string[i]) && (string[i] !='-')))
+	    {
+	      done = 1;
+	      break;
+	    }
 	  else if (string[i] == '\n')
 	    {
 	      int c = string[i + 1];
 
-	      /* Only check for continuation lines is we're in a header. */
+	      /* Only check for continuation lines if we're in a header. */
 	      if (!colon_seen)
 		{
 		  done = 1;
@@ -405,7 +412,7 @@ http_handle_request (HTTP_REQUEST *request, int fd)
 {
   METHOD_HANDLER *mh = http_find_method_handler (request);
   HTTP_RESULT *result = (HTTP_RESULT *)NULL;
-  char *remote_user = pagefunc_get_variable ("mhtml::remote-user");
+  char *remote_ident = pagefunc_get_variable ("mhtml::remote-ident");
   long when = (long) time ((time_t *)0);
 
   int refered_to = 0;
@@ -422,7 +429,7 @@ http_handle_request (HTTP_REQUEST *request, int fd)
     
     bprintf (buffer, "%s %s %s [%s] ",
 	     request->requester,
-	     remote_user ? remote_user : "-",
+	     remote_ident ? remote_ident : "-",
 	     authuser ? authuser : "-",
 	     mhttpd_date_format (when));
 
@@ -585,8 +592,9 @@ mhttpd_handle_empty_page (HTTP_RESULT *result)
       bprintf (result->page, "</body>\n</html>");
     }
 
-  page_insert_http_header (result->page, page_EXPIRES_NOW);
-  bprintf_insert (result->page, 0, "%s\n", result_string);
+  bprintf_insert (result->page, 0, "%s\nContent-type: text/html\n",
+		  result_string);
+  mhttpd_fill_in_missing_headers (result);
 }
 
 #if defined (NOT_USED)
@@ -634,6 +642,7 @@ mr_set_cookie (HTTP_RESULT *result, char *ignore)
 {
   char *answer = (char *)NULL;
   char *temp;
+  char *permanent_p = pagefunc_get_variable ("mhtml::permanent-cookies");
 
   /* Set the SID cookie if that makes sense.
      I think it is important to pass the set-cookie command to each
@@ -641,40 +650,57 @@ mr_set_cookie (HTTP_RESULT *result, char *ignore)
      Then, when they talk back to us, they can pass the cookie back
      in, and we'll detect that they are using cookies, and thus won't
      have to put the SID in the URL.  Cool, huh? */
-  if ( /* mhtml_cookie_compatible && */
-      ((temp = pagefunc_get_variable ("SID")) != (char *)NULL))
-    {
-      BPRINTF_BUFFER *cookie_buffer = bprintf_create_buffer ();
-      char *sid_path = pagefunc_get_variable ("mhtml::sid-prefix");
-      long ticks;
 
-      if (!sid_path)
+  /* First off, only do this if the browser didn't pass the cookie in,
+     or, if the browser did pass the cookie in, if the variable
+     mhtml::permanent-cookies is not set. */
+  temp = pagefunc_get_variable ("mhtml::cookies[0]");
+
+  if (((temp == (char *)NULL) || (strncmp (temp, "SID=", 4) != 0)) ||
+      (permanent_p == (char *)NULL))
+    {
+      if ( /* mhtml_cookie_compatible && */
+	  ((temp = pagefunc_get_variable ("SID")) != (char *)NULL))
 	{
-	  sid_path = pagefunc_get_variable ("mhtml::relative-prefix");
+	  BPRINTF_BUFFER *cookie_buffer = bprintf_create_buffer ();
+	  char *sid_path = pagefunc_get_variable ("mhtml::sid-prefix");
+	  long ticks;
 
 	  if (!sid_path)
-	    sid_path = "/";
-	}
+	    {
+	      sid_path = pagefunc_get_variable ("mhtml::relative-prefix");
 
-      bprintf (cookie_buffer, "SID=%s; path=%s; expires=", temp, sid_path);
+	      if (!sid_path)
+		sid_path = "/";
+	    }
 
-      ticks = (long) time ((time_t *)0);
+	  bprintf (cookie_buffer, "SID=%s; path=%s; expires=", temp, sid_path);
 
-      if (*temp)
-	{
-	  int timeout = 60 * session_get_timeout ((session_id_t) temp);
+	  ticks = (long) time ((time_t *)0);
 
-	  if (timeout > 0)
-	    ticks += timeout;
+	  if (*temp)
+	    {
+	      int timeout = 60 * session_get_timeout ((session_id_t) temp);
+
+	      if (timeout > 0)
+		{
+		  if (permanent_p)
+		    ticks = 946700000;
+		  else
+		    ticks += timeout;
+		}
+	      else
+		{
+		  ticks += 1000;
+		}
+	    }
 	  else
-	    ticks += 1000;
-	}
-      else
-	ticks -= 1000;
+	    ticks -= 1000;
 
-      bprintf (cookie_buffer, "%s", http_date_format (ticks));
-      answer = cookie_buffer->buffer;
-      free (cookie_buffer);
+	  bprintf (cookie_buffer, "%s", http_date_format (ticks));
+	  answer = cookie_buffer->buffer;
+	  free (cookie_buffer);
+	}
     }
 
   return (answer);
@@ -685,7 +711,15 @@ mr_content_length (HTTP_RESULT *result, char *ignore)
 {
   static char num[40];
 
-  sprintf (num, "%d", result->page->bindex);
+  if ((result->page->bindex == 1) &&
+      (result->result_code == res_MOVED_TEMPORARILY))
+    {
+      num[0] = '0';
+      num[1] = '\0';
+    }
+  else
+    sprintf (num, "%d", result->page->bindex);
+
   return (num);
 }
 
@@ -900,11 +934,9 @@ mhttpd_fill_in_missing_headers (HTTP_RESULT *result)
     result->page = page_create_page ();
 
   present = mime_headers_from_string (result->page->buffer, &end_of_headers);
+
   if (end_of_headers)
-    {
-      end_of_headers++;
-      bprintf_delete_range (result->page, 0, end_of_headers);
-    }
+    bprintf_delete_range (result->page, 0, end_of_headers);
 
   /* Next, make sure that the required headers are there. */
   for (i = 0; mime_resolvers[i].mime_name != (char *)NULL; i++)
@@ -1032,17 +1064,23 @@ mhttpd_build_environment (HTTP_RESULT *result)
   add_to_env ("REMOTE_HOST", result->request->requester);
   add_to_env ("REMOTE_ADDR", result->request->requester_addr);
   add_to_env ("REMOTE_USER", pagefunc_get_variable ("mhtml::remote-user"));
+  add_to_env ("REMOTE_IDENT", pagefunc_get_variable ("mhtml::remote-ident"));
   add_to_env
     ("SERVER_PROTOCOL", mhtml_concat
      (3, result->request->protocol, "/", result->request->protocol_version));
 
   add_to_env ("HTTP_AUTHORIZATION",
 	      mhttpd_get_mime_header (headers, "Authorization"));
-  add_to_env ("HTTP_USER_AGENT",
-	      mhttpd_get_mime_header (headers, "User-Agent"));
+
+  if ((char *)getenv ("AUTH_TYPE") != (char *)NULL)
+    add_to_env ("AUTH_TYPE", (char *)getenv ("AUTH_TYPE"));
+
   add_to_env ("PATH_INFO", spec->path_info);
   add_to_env ("REQUEST_METHOD", result->request->method);
 
+  add_to_env ("HTTP_USER_AGENT",
+	      mhttpd_get_mime_header (headers, "User-Agent"));
+
   if (!spec->argc)
     {
       spec->argv = (char **)xmalloc (2 * sizeof (char *));
@@ -1075,6 +1113,31 @@ mhttpd_build_environment (HTTP_RESULT *result)
   if (gbl_passed_sid)
     add_to_env ("SID", gbl_passed_sid);
 
+  /* Add SSL variables if present. */
+  if (environ != (char **)NULL)
+    {
+      register int i;
+      char var_and_val[256];
+      char *temp;
+
+      for (i = 0; environ[i] != (char *)NULL; i++)
+	{
+	  if ((strncmp (environ[i], "HTTP", 4) == 0) &&
+	      (strlen (environ[i]) < 255))
+	    {
+	      strcpy (var_and_val, environ[i]);
+	      temp = strchr (var_and_val, '=');
+
+	      if (temp != (char *)NULL)
+		{
+		  *temp = '\0';
+		  temp++;
+		  add_to_env (var_and_val, temp);
+		}
+	    }
+	}
+    }
+
   add_to_env ("WEBBASEDIR", sv_DocumentRoot);
 }
 
@@ -1112,15 +1175,15 @@ mhtml_read_content_from_fd (HTTP_RESULT *result, int fd)
       diff_mills = ((1000 * (after.tv_sec - before.tv_sec)) +
 		    (after.tv_usec - before.tv_usec));
 
-      /* Now we now how many milliseconds it takes to transfer CONTENT_LENGTH
-	 bytes from the client to this server.  Guess we should set a
-	 variable? */
+      /* Now we know how many milliseconds it takes to transfer
+	 CONTENT_LENGTH bytes from the client to this server.
+	 Guess we should set a variable? */
       {
 	char dbuf[100];
 
 	sprintf (dbuf, "<div %d.0 %lu>",
 		 result->spec->content_length, diff_mills);
-	pagefunc_set_variable ("mhttpd::bytes-per-ms", dbuf);
+	pagefunc_set_variable ("mhtml::bytes-per-ms", dbuf);
       }
 #endif
     }
@@ -1222,6 +1285,26 @@ mhttpd_set_mhtml_variables (HTTP_RESULT *result)
 	    ("mhtml::relative-prefix", physical_path + root_len);
 	}
     }
+  else if (logical_path[1] != '\0')
+    {
+      char *x = strdup (logical_path);
+      temp = strrchr (x, '/');
+      if (temp)
+	{
+	  *temp = '\0';
+	  pagefunc_set_variable ("mhtml::relative-prefix", x);
+	}
+
+      free (x);
+    }
+
+  /* MHTML::PHYSICAL-FILENAME */
+  pagefunc_set_variable ("mhtml::physical-filename", physical_path);
+
+  /* MHTML::PHYSICAL-DIRECTORY */
+  pagefunc_set_variable ("mhtml::physical-directory", physical_path);
+  temp = strrchr (pagefunc_get_variable ("mhtml::physical-directory"), '/');
+  if (temp != (char *)NULL) *temp = '\0';
 
   /* MHTML::LOCATION */
   if (gbl_passed_sid && !mhtml_cookie_compatible)
@@ -1639,6 +1722,7 @@ static void
 mhttpd_metahtml_engine (HTTP_RESULT *result)
 {
   char *prologue_doc = pagefunc_get_variable ("mhtml::prologue-document");
+  char *page_function = pagefunc_get_variable ("mhtml::per-page-function");
   int dont_process = 0;
 
   result->page = page_read_template (result->spec->physical_path);
@@ -1706,6 +1790,26 @@ mhttpd_metahtml_engine (HTTP_RESULT *result)
 	    }
 	}
 
+      /* If there is a per-page-function call, then do it now. */
+      if (!dont_process && (page_function != (char *)NULL))
+	{
+	  PAGE *per_page = page_create_page ();
+	  bprintf (per_page, "<%s> </%s>", page_function, page_function);
+	  page_process_page (per_page);
+
+	  if (per_page != (PAGE *)NULL)
+	    {
+	      if (mhttpd_page_redirect_p (per_page))
+		{
+		  page_free_page (result->page);
+		  result->page = per_page;
+		  dont_process++;
+		}
+	      else
+		page_free_page (per_page);
+	    }
+	}
+
       if (!dont_process)
 	{
 	  char *epilogue_doc =
@@ -1761,11 +1865,10 @@ read_from_fd (int fd)
   timeout.tv_sec = 90;
   timeout.tv_usec = 0;
 
-  FD_ZERO (&read_fds);
-  FD_SET (fd, &read_fds);
-
   while (intr < 2)
     {
+      FD_ZERO (&read_fds);
+      FD_SET (fd, &read_fds);
       select_result = select (fd + 1, (fd_set *)(&read_fds), 0, 0, &timeout);
       if ((select_result == -1) && (errno == EINTR))
 	intr++;
@@ -2054,18 +2157,27 @@ http_head_handler (ReqFunArgs)
       register int i;
       char *temp = result->page->buffer;
       int limit = result->page->bindex;
+      int found = 0;
 
       for (i = 0; i < (limit - 1); i++)
 	if ((temp[i] == '\n') &&
 	    ((temp[i + 1] == '\n')  ||
 	     ((temp[i + 1] == '\r') && (temp[i + 2] == '\n'))))
 	  {
+	    found++;
+
 	    if (temp[i + 1] == '\r') i++;
 
 	    result->page->bindex = i + 2;
 	    temp[i + 2] = '\0';
 	    break;
 	  }
+
+      if (!found)
+	{
+	  result->page->bindex = 0;
+	  result->page->buffer[0] = '\0';
+	}
     }
   return (result);
 }
