@@ -28,6 +28,12 @@
    Santa Barbara, CA
    93101  */
 
+#if defined (HAVE_CONFIG_H)
+#  include <config.h>
+#endif
+
+#define LANGUAGE_DEFINITIONS_FILE 1
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -37,6 +43,7 @@
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/stat.h>
+#include <setjmp.h>
 #include <bprintf/bprintf.h>
 #include <xmalloc/xmalloc.h>
 #include "pages.h"
@@ -227,7 +234,15 @@ mdb_find_file (char *filename)
 {
   register int i;
   MDB_File *file = (MDB_File *)NULL;
-  char *canon_name = pagefunc_canonicalize_file_name (filename);
+  char *incpref = pagefunc_get_variable ("mhtml::include-prefix");
+  char *relpref = pagefunc_get_variable ("mhtml::relative-prefix");
+  char *canon_name;
+
+  if ((filename && incpref) &&
+      (strncmp (filename, incpref, strlen (incpref)) == 0))
+    canon_name = strdup (filename);
+  else
+    canon_name = mhtml_canonicalize_file_name (filename, incpref, relpref);
 
   if (canon_name != (char *)NULL)
     {
@@ -394,6 +409,7 @@ mdb_eval (MDBArgs)
   CANNOT_REDO;
 
   page_debug_clear ();
+  page_syserr_clear ();
   bprintf (page, "%s\n", line);
   syntax_complete = page_check_syntax (page);
 
@@ -418,6 +434,7 @@ mdb_eval (MDBArgs)
   if ((page != (PAGE *)NULL) && (page->buffer != (char *)NULL))
     {
       page_debug_clear ();
+      page_syserr_clear ();
       page_process_page (page);
 
       if (page)
@@ -433,6 +450,12 @@ mdb_eval (MDBArgs)
       page_debug_clear ();
     }
 
+  if (page_syserr_buffer ())
+    {
+      fprintf (stderr, "[SYSTEM-ERROR-OUTPUT]: %s\n", page_syserr_buffer ());
+      page_syserr_clear ();
+    }
+
   return (result);
 }
 
@@ -932,11 +955,11 @@ initialize_page_handlers (void)
   page_process_page (page);
   page_free_page (page);
 
-  sym = symbol_intern_in_package
-    (pagefunc_function_package, MDB_BreakPFunDesc.tag);
+  sym = symbol_intern_in_package 
+    (mhtml_function_package, MDB_BreakPFunDesc.tag);
   sym->values = (char **)&MDB_BreakPFunDesc;
 
-  sym = symbol_intern_in_package (pagefunc_function_package, "INCLUDE");
+  sym = symbol_intern_in_package (mhtml_function_package, "INCLUDE");
   sym->values = (char **)&MDB_IncludePFunDesc;
 }
 
@@ -956,7 +979,7 @@ mdb_breakpoint (MDBArgs)
     return (make_error ("`%s' requires a line number", name));
 
   file = mdb_find_file (filename);
-  
+
   if (!file)
     result = make_error ("Cannot load `%s' for `%s'", name, filename);
   else
@@ -968,7 +991,11 @@ mdb_breakpoint (MDBArgs)
   return (result);
 }
       
-#define INCLUDE_RECURSION_LIMIT 10
+static void
+mdb_apropos (MDBArgs)
+{
+}
+
 static int include_recursive_calls = 0;
 
 static void
@@ -977,15 +1004,21 @@ pf_include_handler (PFunArgs)
   int verbatim_p = var_present_p (vars, "VERBATIM");
 
   include_recursive_calls++;
-  if (include_recursive_calls < INCLUDE_RECURSION_LIMIT)
+  if (include_recursive_calls < MHTML_INCLUDE_RECURSION_LIMIT)
     {
-      char *arg = page_evaluate_string (get_positional_arg (vars, 0));
-      char *alt = page_evaluate_string
+      char *arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
+      char *alt = mhtml_evaluate_string
 	(get_one_of (vars, "ALT", "ALTERNATE", (char *)0));
       char *canonicalized_name = (char *)NULL;
+      char *incpref = pagefunc_get_variable ("%%::incpref");
+      char *relpref = pagefunc_get_variable ("%%::relpref");
+
+      if (!incpref) incpref = pagefunc_get_variable ("mhtml::include-prefix");
+      if (!relpref) relpref = pagefunc_get_variable ("mhtml::relative-prefix");
 
       if (arg != (char *)NULL)
-	canonicalized_name = pagefunc_canonicalize_file_name (arg);
+	canonicalized_name =
+	  mhtml_canonicalize_file_name (arg, incpref, relpref);
 
       if (canonicalized_name != (char *)NULL)
 	{
@@ -1017,9 +1050,50 @@ pf_include_handler (PFunArgs)
 
 	  if (file_contents)
 	    {
-	      bprintf_insert (page, start, "%s", file_contents->buffer);
+	      /* Manually insert the file instead of letting bprintf
+		 do it for us.  This is because the file could contain
+		 binary data, and then file->bindex wouldn't necessarily
+		 reflect the length of what was inserted. */
+	      if ((file_contents->bindex + page->bindex) >= page->bsize)
+		page->buffer = (char *)xrealloc
+		(page->buffer, (page->bsize += (file_contents->bindex + 100)));
+
+	      memmove (page->buffer + start + file_contents->bindex,
+		       page->buffer + start,
+		       (page->bindex + 1) - start);
+
+	      memcpy (page->buffer + start, file_contents->buffer,
+		      file_contents->bindex);
+	      page->bindex += file_contents->bindex;
+
 	      if (verbatim_p)
 		*newstart += file_contents->bindex;
+#if defined (MHTML_INCLUDE_IS_RELATIVE)
+	      else if (!empty_string_p (relpref))
+		{
+		  char *temp = strstr (canonicalized_name, incpref);
+
+		  if (temp != (char *)NULL)
+		    {
+		      char *last;
+
+		      temp += strlen (incpref);
+		      last = strrchr (temp, '/');
+
+		      if (last != (char *)NULL)
+			{
+			  *last = '\0';
+
+			  bprintf_insert
+			    (page, start + file_contents->bindex,
+			     "<set-var %%::relpref=%s>", relpref);
+
+			  pagefunc_set_variable ("%%::relpref", temp);
+
+			}
+		    }
+		}
+#endif /* MHTML_INCLUDE_IS_RELATIVE */
 
 	      page_free_page (file_contents);
 	    }
@@ -1028,3 +1102,4 @@ pf_include_handler (PFunArgs)
     }
   include_recursive_calls--;
 }
+
