@@ -28,6 +28,14 @@
    Santa Barbara, CA
    93101  */
 
+#if defined (HAVE_CONFIG_H)
+#  include <config.h>
+#endif
+
+#if !defined (MHTML_SYSTEM_TYPE)
+#  define MHTML_SYSTEM_TYPE "Incorrectly Compiled"
+#endif
+
 #define LANGUAGE_DEFINITIONS_FILE 1
 
 #include <stdio.h>
@@ -82,6 +90,7 @@ main (int argc, char *argv[])
   printf ("MDB 2.0: Interactive Debugger for Meta-HTMLv%s, by Brian J. Fox.",
 	  mhtml_version_string);
   printf ("%s\n", metahtml_copyright_string);
+  printf ("Compiled for (%s)\n", MHTML_SYSTEM_TYPE);
   printf ("CAUTION!  This is experimental software; bugs certainly exist.\n");
 
   initialize_mdb (argc, argv);
@@ -204,6 +213,7 @@ initialize_mdb (int argc, char *argv[])
   mdb_initialize_readline ();
 
   pagefunc_set_variable ("mhtml::version", mhtml_version_string);
+  pagefunc_set_variable ("mhtml::system-type", MHTML_SYSTEM_TYPE);
   pagefunc_set_variable ("mhtml::path-info", "");
   pagefunc_set_variable ("mhtml::http-to-host", "");
   pagefunc_set_variable ("mhtml::current-doc", "");
