@@ -28,6 +28,14 @@
    Santa Barbara, CA
    93101  */
 
+#if defined (HAVE_CONFIG_H)
+#  include <config.h>
+#endif
+
+#if !defined (MHTML_SYSTEM_TYPE)
+#  define MHTML_SYSTEM_TYPE "Incorrectly Compiled"
+#endif
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -40,7 +48,7 @@
 #include <xmalloc/xmalloc.h>
 #include "pages.h"
 #include "session_data.h"
-#include "pagefuncs.h"
+#include "parser.h"
 
 #if !defined (MHTML_VERSION_STRING)
 static char *mhtml_version_string = "";
@@ -133,6 +141,8 @@ prep_for_page_process (PAGE *page, int argc, char *argv[])
 	}
 
       pagefunc_set_variable ("mhtml::version", mhtml_version_string);
+      pagefunc_set_variable ("mhtml::system-type", MHTML_SYSTEM_TYPE);
+      pagefunc_set_variable ("mhtml::exec-path", (char *)getenv ("PATH"));
 
 #if !defined (macintosh)
       /* Now set the environment variables. */
@@ -211,6 +221,49 @@ parse_program_args (int argc, char *argv[])
     usage ();
   else if (filename)
     input_contents = page_read_template (filename);
+
+  {
+    char *temp = pagefunc_get_variable ("mhtml::include-prefix");
+
+    if (empty_string_p (temp))
+      {
+	temp = strrchr (filename, '/');
+
+	if (temp != (char *)NULL)
+	  {
+	    *temp = '\0';
+	    pagefunc_set_variable ("mhtml::include-prefix", filename);
+	  }
+	else
+	  {
+	    char dir[1024];
+	    temp = getcwd (dir, sizeof (dir));
+	    temp = strrchr (dir, '/');
+	    if (temp != (char *)NULL)
+	      {
+		*temp = '\0';
+		pagefunc_set_variable ("mhtml::include-prefix", dir);
+	      }
+	  }
+      }
+
+    /* Define default functions here. */
+    {
+      PAGE *p = page_create_page ();
+
+      bprintf (p, "<defsubst meta-html>");
+      bprintf (p, "<font color=\"#00309B\">Meta-</font>HTML");
+      bprintf (p, "<if <not <get-var mhtml::printed-tm>>");
+      bprintf (p, "<prog <sup><font size=-2>TM</font></sup>");
+      bprintf (p, "<increment mhtml::printed-tm>>>");
+      bprintf (p, "</defsubst>");
+      bprintf (p, "<defmacro ptext>");
+      bprintf (p, "<plain-text first-char=<font size=+2> nobr=true>");
+      bprintf (p, "%s", "%body</plain-text></defmacro>");
+      page_process_page (p);
+      page_free_page (p);
+    }
+  }
 }
 
 #if defined (CHECK_TIMESTAMP)
@@ -232,3 +285,4 @@ check_timestamp (char *filename)
     }
 }
 #endif /* CHECK_TIMESTAMP */
+
