@@ -32,6 +32,12 @@
 #  include <config.h>
 #endif
 
+#if !defined (MHTML_SYSTEM_TYPE)
+#  define MHTML_SYSTEM_TYPE "Incorrectly Compiled"
+#endif
+
+#define I_UNDERSTAND_IDENT
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -75,6 +81,10 @@ extern int open (char *, int, ...);
 #include "globals.h"
 #include "logging.h"
 
+#if defined (HAVE_GETPWNAM)
+#  include <pwd.h>
+#endif
+
 #if !defined (errno)
 extern int errno;
 #endif
@@ -100,8 +110,10 @@ static int forced_port_number = 0;
 /* The port number that we normally connect on. */
 static int http_port = 80;
 
+#if defined (MHTTPD_HTML_ADMINISTRATION)
 /* The port number that we connect on for administration. */
 static int admin_port = 8008;
+#endif
 
 /* When non-null, this is the hostname of the IP address that we wish to
    bind our socket tightly to.  If not set, it defaults to any available
@@ -126,6 +138,9 @@ static int default_limit = 120;
    address. */
 static int servers_to_fork = 0;
 
+/* Non-zero means run once under inetd, and exit. */
+static int once_and_exit = 0;
+
 /* Forward declarations. */
 static void parse_program_args (int argc, char *argv[]);
 static void loop_over_connections (void);
@@ -149,14 +164,22 @@ main (int argc, char *argv[])
   check_timestamp (argv[0]);
 #endif
 
-  fprintf (stderr, "%s: Initializing server...", argv[0]); 
-  fflush (stderr);
-  initialize_server ();
-  fprintf (stderr, "done.\n");
-  fflush (stderr);
-  fflush (stdout);
+  if (!once_and_exit)
+    {
+      fprintf (stderr, "%s: Initializing server...", argv[0]); 
+      fflush (stderr);
+    }
 
-  if (debug_mode)
+  initialize_server ();
+
+  if (!once_and_exit)
+    {
+      fprintf (stderr, "done.\n");
+      fflush (stderr);
+      fflush (stdout);
+    }
+
+  if (debug_mode || once_and_exit)
     loop_over_connections ();
   else
     {
@@ -270,10 +293,13 @@ static char *deletable_variables[] =
   "mhtml::agent-log",
   "mhtml::debug-log",
   "mhtml::log-performance",
+  "mhtml::default-user",
+#if defined (MHTTPD_HTML_ADMINISTRATION)
   "mhtml::admin-root",
+#endif
   "mhttpd::per-request-function",
   "mhttpd::session-database-file",
-  "mhttpd::get-remote-user",
+  "mhttpd::get-remote-ident",
 
   (char *)NULL
 };
@@ -316,6 +342,41 @@ reinitialize_server (void)
   if (!page)
     fatal ("Cannot read configuration file `%s'!", mhttpd_config_path);
 
+  if (once_and_exit)
+    {
+      /* Set up the input and output file descriptors. */
+      mhtml_stdout_fileno = fileno (stdout);
+      mhtml_stdin_fileno = fileno (stdin);
+      mhtml_stderr_fileno = fileno (stderr);
+
+      /* Quickly make a package containing the minimum mime-types. */
+      pagefunc_set_variable ("mime-type::.mhtml", "metahtml/interpreted");
+      pagefunc_set_variable ("mime-type::.jpeg", "image/jpeg");
+      pagefunc_set_variable ("mime-type::.jpg", "image/jpeg");
+      pagefunc_set_variable ("mime-type::.gif", "image/gif");
+      pagefunc_set_variable ("mime-type::.html", "text/html");
+      pagefunc_set_variable ("mime-type::.txt", "text/plain");
+      pagefunc_set_variable ("mime-type::.mov", "video/quicktime");
+      pagefunc_set_variable ("mime-type::.default", "text/plain");
+
+      /* The minimum startup documents. */
+      pagefunc_set_variable ("mhtml::default-filenames[]",
+			     "Welcome.mhtml\nwelcome.mhtml\n\
+Index.mhtml\nindex.mhtml\n\
+Home.mhtml\nhome.mhtml\n\
+Directory.mhtml\ndirectory.mhtml\n\
+index.html\nhome.html");
+
+      /* Set mhtml::require-directories[], to a reasonable value here. */
+      pagefunc_set_variable ("mhtml::require-directories[]",
+".\ntagsets\nmacros\nincludes\n\
+..\n../tagsets\n../macros\n../includes\n\
+../..\n../../tagsets\n../../macros\n../../includes");
+
+      /* The default extensions for running files through the engine. */
+      pagefunc_set_variable ("mhtml::metahtml-extensions[]", ".mhtml");
+    }
+
   if (page->buffer)
     page_process_page (page);
 
@@ -340,7 +401,7 @@ reinitialize_server (void)
       pagefunc_set_variable ("mhtml::server-name", http_host);
       http_host_set = 0;
     }
-      
+
   {
     int val = 0;
 
@@ -375,11 +436,13 @@ reinitialize_server (void)
 	pagefunc_set_variable ("mhtml::server-port", num);
       }
 
+#if defined (MHTTPD_HTML_ADMINISTRATION)
     admin_port = 0;
 
     temp = pagefunc_get_variable ("mhttpd::admin-port");
     if ((temp != (char *)NULL) && ((val = atoi (temp)) > 0))
       admin_port = val;
+#endif /* MHTTPD_HTML_ADMINISTRATION */
   }
 
   temp = pagefunc_get_variable ("mhtml::document-root");
@@ -451,7 +514,8 @@ reinitialize_server (void)
   else
     mhttpd_log (log_ERROR, "Finshed reading %s again...", mhttpd_config_path);
 
-  signal (SIGHUP, (sig_t) reinitialize_server);
+  if (!once_and_exit)
+    signal (SIGHUP, (sig_t) reinitialize_server);
 }
 
 static int
@@ -527,6 +591,7 @@ initialize_server ()
   if (!mhttpd_config_path)
     fatal ("Cannot find mhttpd.conf!");
 
+  pagefunc_set_variable ("mhtml::system-type", MHTML_SYSTEM_TYPE);
   reinitialize_server ();
 
   if (mhttpd_debugging)
@@ -563,6 +628,8 @@ parse_program_args (int argc, char *argv[])
 	}
       else if (strcasecmp (arg,  "--ssl") == 0)
 	mhttpd_ssl_server = 1;
+      else if (strcasecmp (arg, "--inetd") == 0)
+	once_and_exit = 1;
       else if ((strcasecmp (arg, "--config") == 0) ||
 	       (strcasecmp (arg, "-config") == 0) ||
 	       (strcasecmp (arg, "-f") == 0))
@@ -832,6 +899,7 @@ setup_signal_handling (void)
   signal (SIGCHLD, (sig_t) reap_children);
 }
 
+#if defined (MHTTPD_HTML_ADMINISTRATION)
 /* Called in the child just after connection.  This modifies globals
    to reflect the different document root, etc. */
 static void
@@ -870,14 +938,27 @@ set_administration_variables (void)
   mhttpd_log_agent = 0;
   mhttpd_per_request_function = (char *)NULL;
 }
+#endif /* MHTTPD_HTML_ADMINISTRATION */
+
+#if defined (I_UNDERSTAND_IDENT)
+static void
+connect_timed_out (void)
+{
+  alarm (0);
+}
+#endif
 
 static void
 loop_over_connections (void)
 {
-  int sock = -1, admin_sock = -1;
+  int sock = -1;
   int sock_size = sizeof (struct sockaddr_in);
-  struct sockaddr_in socket_address, admin_address;
   struct in_addr host_address;
+  struct sockaddr_in socket_address;
+#if defined (MHTTPD_HTML_ADMINISTRATION)  
+  int admin_sock = -1;
+  struct sockaddr_in admin_address;
+#endif
 
   host_address.s_addr = htonl (INADDR_ANY);
 
@@ -892,7 +973,7 @@ loop_over_connections (void)
       memcpy (&host_address, addr, sizeof (host_address));
     }
 
-  if (!debug_mode)
+  if (!debug_mode && !once_and_exit)
     {
       int one = 1;
 
@@ -912,6 +993,7 @@ loop_over_connections (void)
 		sizeof (socket_address)) == -1)
 	fatal ("Cannot bind socket to socket address");
 
+#if defined (MHTTPD_HTML_ADMINISTRATION)
       if (admin_port != 0)
 	{
 	  memcpy (&admin_address, &socket_address,
@@ -927,6 +1009,7 @@ loop_over_connections (void)
 		    sizeof (socket_address)) == -1)
 	    fatal ("Cannot bind administration socket");
 	}
+#endif /* MHTTPD_HTML_ADMINISTRATION */
 
       if (servers_to_fork)
 	{
@@ -941,18 +1024,23 @@ loop_over_connections (void)
       /* listen (sock, (servers_to_fork > 1) ? 1 : 5); */
       listen (sock, 512);
 
+#if defined (MHTTPD_HTML_ADMINISTRATION)
       if (admin_port != 0)
 	listen (admin_sock, 10);
+#endif
     }
 
   while (1)
     {
       int connection = -1;
       pid_t child = (pid_t)0;
-      int administration = 0;
       int high_fd = sock;
 
-      if (!debug_mode)
+#if defined (MHTTPD_HTML_ADMINISTRATION)
+      int administration = 0;
+#endif
+
+      if (!debug_mode && !once_and_exit)
 	{
 	  struct timeval nohang = { 0, 0 };
 	  fd_set read_fds;
@@ -964,11 +1052,13 @@ loop_over_connections (void)
 	      FD_ZERO (&read_fds);
 	      FD_SET (sock, &read_fds);
 
+#if defined (MHTTPD_HTML_ADMINISTRATION)
 	      if (admin_port != 0)
 		{
 		  FD_SET (admin_sock, &read_fds);
 		  high_fd = admin_sock;
 		}
+#endif /* MHTTPD_HTML_ADMINISTRATION */
 
 	      ready = select (high_fd + 1, &read_fds,
 			      (fd_set *)0, (fd_set *)0, &nohang);
@@ -981,25 +1071,33 @@ loop_over_connections (void)
 		  FD_ZERO (&read_fds);
 		  FD_SET (sock, &read_fds);
 
+#if defined (MHTTPD_HTML_ADMINISTRATION)
 		  if (admin_port != 0)
 		    FD_SET (admin_sock, &read_fds);
+#endif /* MHTTPD_HTML_ADMINISTRATION */
 
 		  ready = select (high_fd + 1, &read_fds,
 				  (fd_set *)0, (fd_set *)0, &period);
 		}
 
 	      if ((ready > 0) &&
-		  ((FD_ISSET (sock, &read_fds)) ||
-		   ((admin_port != 0) && (FD_ISSET (admin_sock, &read_fds)))))
+		  ((FD_ISSET (sock, &read_fds))
+#if defined (MHTTPD_HTML_ADMINISTRATION)
+		   || ((admin_port != 0) && (FD_ISSET (admin_sock, &read_fds)))
+#endif /* MHTTPD_HTML_ADMINISTRATION */
+		   ))
 		break;
 	    }
 
 	  if (FD_ISSET (sock, &read_fds))
 	    {
+#if defined (MHTTPD_HTML_ADMINISTRATION)
 	      administration = 0;
+#endif /* MHTTPD_HTML_ADMINISTRATION */
 	      connection = accept (sock, (struct sockaddr *)&socket_address,
 				   &sock_size);
 	    }
+#if defined (MHTTPD_HTML_ADMINISTRATION)
 	  else if (admin_port != 0)
 	    {
 	      administration = 1;
@@ -1007,6 +1105,7 @@ loop_over_connections (void)
 		accept (admin_sock, (struct sockaddr *)&admin_address,
 			&sock_size);
 	    }
+#endif /* MHTTPD_HTML_ADMINISTRATION */
 
 	  if (connection < 0)
 	    continue;
@@ -1045,14 +1144,31 @@ loop_over_connections (void)
 	  signal (SIGCHLD, SIG_DFL);
 	  signal (SIGHUP,  SIG_IGN);
 
-	  if (!debug_mode)
+	  if (!debug_mode && !once_and_exit)
 	    {
 	      /* If this connection is SSL, negotiate the security now. */
 	      if (mhttpd_ssl_server)
 		mhttpd_negotiate_ssl (connection);
 
+#if defined (MHTTPD_HTML_ADMINISTRATION)
 	      if (administration)
 		set_administration_variables ();
+	      else
+#endif /* MHTTPD_HTML_ADMINISTRATION */
+		/* Set the user permissions of this server if specified. */
+		{
+#if defined (HAVE_GETPWNAM)
+		  char *user = pagefunc_get_variable ("mhtml::default-user");
+
+		  if (!empty_string_p (user))
+		    {
+		      struct passwd *entry = (struct passwd *)getpwnam (user);
+
+		      if (entry)
+			setuid (entry->pw_uid);
+		    }
+#endif /* HAVE_GETPWNAM */
+		}
 	    }
 	  else
 	    connection = fileno (stdin);
@@ -1131,10 +1247,9 @@ loop_over_connections (void)
 		  else
 		    req->requester = strdup (addr_rep);
 
-#define I_UNDERSTAND_IDENT
 #if defined (I_UNDERSTAND_IDENT)
 		  /* Do the IDENT lookup if that is requested. */
-		  if (pagefunc_get_variable ("mhttpd::get-remote-user"))
+		  if (pagefunc_get_variable ("mhttpd::get-remote-ident"))
 		    {
 		      int fd = -1;
 
@@ -1161,16 +1276,26 @@ loop_over_connections (void)
 
 			if (ident_fd > 0)
 			  {
+			    int error = 0;
 			    setsockopt (ident_fd, SOL_SOCKET, SO_REUSEADDR,
 					(char *)&one, sizeof (one));
 
 			    /* Bind the socket to our address. */
-			    if ((bind (ident_fd, (struct sockaddr *)&local,
-				       sizeof (local)) != -1) &&
-				(connect (ident_fd,
-					  (struct sockaddr *)&remote,
-					  sizeof (remote)) != -1))
-			      fd = ident_fd;
+			    error = bind (ident_fd, (struct sockaddr *)&local,
+					  sizeof (local));
+			    if (error != -1)
+			      {
+				signal (SIGALRM, (sig_t)connect_timed_out);
+				alarm (10);
+				error = connect (ident_fd,
+						 (struct sockaddr *)&remote,
+						 sizeof (remote));
+				alarm (0);
+				signal (SIGALRM, SIG_DFL);
+
+				if (error != -1)
+				  fd = ident_fd;
+			      }
 			  }
 		      }
 #endif
@@ -1209,7 +1334,7 @@ loop_over_connections (void)
 				    break;
 				  }
 			      pagefunc_set_variable_readonly
-				("mhtml::remote-user", comm + i);
+				("mhtml::remote-ident", comm + i);
 			    }
 			  mhttpd_log (log_DEBUG, "Got %s", comm);
 			  shutdown (fd, 2);
@@ -1221,19 +1346,20 @@ loop_over_connections (void)
 	      know_my_client = 1;
 	    }
 
+	  mhttpd_mime_headers_to_package
+	    (req->headers, "mhttpd-received-headers");
+	  pagefunc_set_variable ("mhttpd::method", req->method);
+	  pagefunc_set_variable ("mhttpd::protocol", req->protocol);
+	  pagefunc_set_variable ("mhttpd::protocol-version",
+				 req->protocol_version);
+	  pagefunc_set_variable ("mhttpd::location", req->location);
+	  pagefunc_set_variable ("mhttpd::requester", req->requester);
+	  pagefunc_set_variable ("mhttpd::requester-addr",
+				 req->requester_addr);
+
 	  if (req && mhttpd_per_request_function)
 	    {
 	      PAGE *perfun = page_create_page ();
-	      mhttpd_mime_headers_to_package
-		(req->headers, "mhttpd-received-headers");
-	      pagefunc_set_variable ("mhttpd::method", req->method);
-	      pagefunc_set_variable ("mhttpd::protocol", req->protocol);
-	      pagefunc_set_variable ("mhttpd::protocol-version",
-				     req->protocol_version);
-	      pagefunc_set_variable ("mhttpd::location", req->location);
-	      pagefunc_set_variable ("mhttpd::requester", req->requester);
-	      pagefunc_set_variable ("mhttpd::requester-addr",
-				     req->requester_addr);
 	      bprintf (perfun, "<%s>\n", mhttpd_per_request_function);
 	      page_process_page (perfun);
 
